@@ -1,5 +1,5 @@
 import copy
-import time
+import datetime
 import uuid
 from functools import partial
 from typing import Any, Dict, Type, TypeVar
@@ -21,7 +21,8 @@ _db_hosts: Dict[str, Any] = {}
 
 
 def default_timestamp() -> int:
-    return int(time.time())
+    now = datetime.datetime.utcnow()
+    return int(round(now.timestamp() * 1000))
 
 
 def _get_v1_host(stage: str) -> str:
@@ -64,6 +65,7 @@ def from_collection(
     collection: str,
     stage: str = "stg",
     alias: str = "default",
+    offline: bool = False,
     **meta: Any,
 ) -> Type[BaseDocument]:
     def save_with_updated(self: BaseDocument, *args: Any, **kwargs: Any) -> Any:
@@ -71,17 +73,21 @@ def from_collection(
         return DynamicDocument.save(self, *args, **kwargs)
 
     def unique_key() -> str:
-        return uuid.uuid4().__str__()
+        return uuid.uuid4().__str__()  # pragma: nocover
 
-    host = _get_v1_host(stage)
-    url = f"{host}/collection/{collection}?client_id={client_id}&client_secret={client_secret}"
+    if not offline:
+        host = _get_v1_host(stage)
+        url = f"{host}/collection/{collection}?client_id={client_id}&client_secret={client_secret}"
+
+        resp = requests.get(url)
+        resp.raise_for_status()
+        resp_json = resp.json()
+
+        pk_name = resp_json["primary_key"]
+    else:
+        pk_name = meta.pop("default_pk", None)
 
     alias = f"{client_id}__{alias}"
-
-    resp = requests.get(url)
-    resp.raise_for_status()
-    resp_json = resp.json()
-
     connect(client_id, client_secret, stage=stage, alias=alias)
 
     namespace = {
@@ -91,7 +97,7 @@ def from_collection(
             "db_alias": alias,
             "id_field": "_obj_id",
         },
-        resp_json["primary_key"]: StringField(unique=True, default=unique_key),
+        pk_name: StringField(unique=True, default=unique_key),
         "_obj_id": ObjectIdField(default=ObjectId, primary_key=True),
         "created": IntField(default=default_timestamp),
         "updated": IntField(default=default_timestamp),
